﻿namespace SddpViewer;

using System;
using System.Threading.Tasks;

using CommunityToolkit.Mvvm.ComponentModel;

using Rssdp;

public partial class DiscoveredDeviceViewModel : ObservableObject
{
    private readonly DiscoveredSsdpDevice _device;

    public DiscoveredDeviceViewModel(DiscoveredSsdpDevice device)
    {
        _device = device;
        ResponseHeader = GetResponseHeader();
    }

    private string GetResponseHeader()
    {
        return string.Join(
            "," + Environment.NewLine,
            _device.ResponseHeaders.Select(x => $"{{{x.Key} : {string.Join(";", x.Value)}}}")
        );
    }

    public string ResponseHeader { get; }

    /// <summary>
    /// Sets or returns the type of notification, being either a uuid, device type, service type or upnp:rootdevice.
    /// </summary>
    public string NotificationType => _device.NotificationType;

    /// <summary>
    /// Sets or returns the universal service name (USN) of the device.
    /// </summary>
    public string Usn => _device.Usn;

    /// <summary>
    /// Sets or returns a URL pointing to the device description document for this device.
    /// </summary>
    public Uri DescriptionLocation => _device.DescriptionLocation;

    /// <summary>
    /// Sets or returns the length of time this information is valid for (from the <see cref="P:Rssdp.DiscoveredSsdpDevice.AsAt" /> time).
    /// </summary>
    public TimeSpan CacheLifetime => _device.CacheLifetime;

    /// <summary>
    /// Sets or returns the date and time this information was received.
    /// </summary>
    public DateTimeOffset AsAt => _device.AsAt;

    [ObservableProperty]
    private string _friendlyName = "";

    [ObservableProperty]
    private SsdpDeviceIcon? _icon;

    [ObservableProperty]
    private Uri? _presentationUrl;

    [ObservableProperty]
    private string _modelNumber = "";

    public async Task GetFurtherInformationAsync()
    {
        var ssdpDevice = await _device.GetDeviceInfo().ConfigureAwait(false);
        FriendlyName = ssdpDevice.FriendlyName;
        Icon = ssdpDevice.Icons.MinBy(x => x.Height);
        PresentationUrl = ssdpDevice.PresentationUrl;
        ModelNumber = ssdpDevice.ModelNumber;
    }
}
